# Created by: PyQt6 UI code generator 6.2.3
from PyQt6.QtCore import QSize, QRect, Qt, QCoreApplication, QMetaObject
from PyQt6.QtWidgets import QSizePolicy, QWidget, QVBoxLayout, QCheckBox, \
    QGridLayout, QLabel, QFrame, QGroupBox, QLineEdit, QComboBox, QListWidget, QPushButton,\
    QHBoxLayout, QRadioButton, QButtonGroup, QTabWidget
from PyQt6.QtGui import QFont, QCursor
from matplotlib.backends.qt_compat import QtWidgets
from matplotlib.backends.backend_qtagg import (
    FigureCanvas, NavigationToolbar2QT as NavigationToolbar)
from matplotlib.figure import Figure
from vaspplotsuite.static import font
from os import name as oname


class DosAppWindow(object):

    if oname == "nt":
        q = 0.8
    else:
        q = 1
    font1 = QFont(font[0])
    font1.setPointSize(int(13*q))
    font1.setWeight(75)
    font2 = QFont(font[0])
    font2.setPointSize(int(13*q))
    font2.setWeight(50)
    font3 = QFont(font[0])
    font3.setPointSize(int(13*q))

    def setupUi(self, main_window):
        # MAIN WINDOW INIT
        main_window.setObjectName("main_window")
        main_window.resize(1200, 640)
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(main_window.sizePolicy().hasHeightForWidth())
        main_window.setSizePolicy(sizePolicy)
        main_window.setMinimumSize(QSize(1200, 640))
        main_window.setMaximumSize(QSize(1200, 640))
        main_window.setAutoFillBackground(False)

        # SPIN SELECTION BOX
        self.spin_box = QGroupBox(main_window)
        self.spin_box.setGeometry(QRect(10, 250, 191, 61))
        self.spin_box.setFont(self.font1)
        self.spin_box.setFlat(False)
        self.spin_box.setCheckable(False)
        self.spin_box.setChecked(False)
        self.spin_box.setObjectName("spin_box")
        self.layoutWidget = QWidget(self.spin_box)
        self.layoutWidget.setGeometry(QRect(0, 30, 191, 20))
        self.layoutWidget.setObjectName("layoutWidget")
        self.spin_layout = QHBoxLayout(self.layoutWidget)
        self.spin_layout.setContentsMargins(0, 0, 0, 0)
        self.spin_layout.setObjectName("spin_layout")
        self.spin_both_btn = QRadioButton(self.layoutWidget)
        self.spin_both_btn.setFont(self.font2)
        self.spin_both_btn.setCheckable(True)
        self.spin_both_btn.setChecked(False)
        self.spin_both_btn.setObjectName("spin_both_btn")
        self.spin_layout.addWidget(self.spin_both_btn)
        self.spin_up_btn = QRadioButton(self.layoutWidget)
        self.spin_up_btn.setFont(self.font2)
        self.spin_up_btn.setCheckable(True)
        self.spin_up_btn.setChecked(False)
        self.spin_up_btn.setAutoExclusive(True)
        self.spin_up_btn.setObjectName("spin_up_btn")
        self.spin_layout.addWidget(self.spin_up_btn)
        self.spin_down_btn = QRadioButton(self.layoutWidget)
        self.spin_down_btn.setFont(self.font2)
        self.spin_down_btn.setCheckable(True)
        self.spin_down_btn.setChecked(False)
        self.spin_down_btn.setObjectName("spin_down_btn")
        self.spin_layout.addWidget(self.spin_down_btn)
        self.spin_btn_group = QButtonGroup()
        self.spin_btn_group.addButton(self.spin_both_btn)
        self.spin_btn_group.addButton(self.spin_up_btn)
        self.spin_btn_group.addButton(self.spin_down_btn)
        self.spin_btn_list = [self.spin_both_btn, self.spin_up_btn, self.spin_down_btn]

        # LOAD VASP DATA BOX
        self.load_box = QGroupBox(main_window)
        self.load_box.setGeometry(QRect(10, 10, 411, 91))
        self.load_box.setFont(self.font1)
        self.load_box.setObjectName("load_box")
        self.verticalLayout = QVBoxLayout(self.load_box)
        self.verticalLayout.setObjectName("verticalLayout")
        self.load_layout = QHBoxLayout()
        self.load_layout.setObjectName("load_layout")
        self.load_txt = QLineEdit(self.load_box)
        self.load_txt.setFont(self.font2)
        self.load_txt.setObjectName("load_txt")
        self.load_layout.addWidget(self.load_txt)
        self.browse_btn = QPushButton(self.load_box)
        self.browse_btn.setFont(self.font2)
        self.browse_btn.setObjectName("browse_btn")
        self.load_layout.addWidget(self.browse_btn)
        self.load_btn = QPushButton(self.load_box)
        self.load_btn.setFont(self.font2)
        self.load_btn.setObjectName("load_btn")
        self.load_layout.addWidget(self.load_btn)
        self.verticalLayout.addLayout(self.load_layout)

        # ATOM SELECTION BOX
        self.atom_sel_box = QGroupBox(main_window)
        self.atom_sel_box.setGeometry(QRect(10, 150, 191, 91))
        self.atom_sel_box.setFont(self.font1)
        self.atom_sel_box.setObjectName("atom_sel_box")
        self.atom_tabs = QTabWidget(self.atom_sel_box)
        self.atom_tabs.setGeometry(QRect(0, 20, 191, 71))
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.atom_tabs.sizePolicy().hasHeightForWidth())
        self.atom_tabs.setSizePolicy(sizePolicy)
        self.atom_tabs.setFont(self.font2)
        self.atom_tabs.setAutoFillBackground(False)
        self.atom_tabs.setTabShape(QTabWidget.TabShape.Rounded)
        self.atom_tabs.setIconSize(QSize(16, 16))
        self.atom_tabs.setTabsClosable(False)
        self.atom_tabs.setMovable(False)
        self.atom_tabs.setTabBarAutoHide(False)
        self.atom_tabs.setObjectName("atom_tab")
        self.atom_sel_tab = QWidget()
        self.atom_sel_tab.setObjectName("atom_sel_tab")
        self.atom_comb = QComboBox(self.atom_sel_tab)
        self.atom_comb.setGeometry(QRect(20, 10, 141, 26))
        self.atom_comb.setObjectName("atom_comb")
        self.atom_tabs.addTab(self.atom_sel_tab, "")
        self.atom_list_tab = QWidget()
        self.atom_list_tab.setObjectName("atom_list_tab")
        self.atom_text = QLineEdit(self.atom_list_tab)
        self.atom_text.setGeometry(QRect(10, 10, 161, 21))
        self.atom_text.setText("")
        self.atom_text.setObjectName("atom_text")
        self.atom_tabs.addTab(self.atom_list_tab, "")

        # STATES SELECTION BOX
        self.states_box = QGroupBox(main_window)
        self.states_box.setGeometry(QRect(220, 150, 201, 261))
        self.states_box.setFont(self.font1)
        self.states_box.setObjectName("states_box")
        self.states_tabs = QTabWidget(self.states_box)
        self.states_tabs.setGeometry(QRect(-1, 19, 201, 241))
        self.states_tabs.setFont(self.font2)
        self.states_tabs.setObjectName("states_tabs")
        # SUBSHELL TAB
        self.subshell_tab = QWidget()
        self.subshell_tab.setObjectName("subshell_tab")
        self.layoutWidget1 = QWidget(self.subshell_tab)
        self.layoutWidget1.setGeometry(QRect(80, 40, 41, 131))
        self.layoutWidget1.setObjectName("layoutWidget1")
        self.subshell_layout = QVBoxLayout(self.layoutWidget1)
        self.subshell_layout.setContentsMargins(0, 0, 0, 0)
        self.subshell_layout.setObjectName("subshell_layout")
        self.ss_box = QCheckBox(self.layoutWidget1)
        self.ss_box.setFont(self.font2)
        self.ss_box.setCursor(QCursor(Qt.CursorShape.ArrowCursor))
        self.ss_box.setCheckable(True)
        self.ss_box.setChecked(False)
        self.ss_box.setTristate(False)
        self.ss_box.setObjectName("ss_box")
        self.subshell_layout.addWidget(self.ss_box)
        self.pp_box = QCheckBox(self.layoutWidget1)
        self.pp_box.setFont(self.font2)
        self.pp_box.setCursor(QCursor(Qt.CursorShape.ArrowCursor))
        self.pp_box.setCheckable(True)
        self.pp_box.setChecked(False)
        self.pp_box.setTristate(False)
        self.pp_box.setObjectName("pp_box")
        self.subshell_layout.addWidget(self.pp_box)
        self.dd_box = QCheckBox(self.layoutWidget1)
        self.dd_box.setFont(self.font2)
        self.dd_box.setCursor(QCursor(Qt.CursorShape.ArrowCursor))
        self.dd_box.setCheckable(True)
        self.dd_box.setChecked(False)
        self.dd_box.setTristate(False)
        self.dd_box.setObjectName("dd_box")
        self.subshell_layout.addWidget(self.dd_box)
        self.ff_box = QCheckBox(self.layoutWidget1)
        self.ff_box.setFont(self.font2)
        self.ff_box.setCursor(QCursor(Qt.CursorShape.ArrowCursor))
        self.ff_box.setCheckable(True)
        self.ff_box.setChecked(False)
        self.ff_box.setTristate(False)
        self.ff_box.setObjectName("ff_box")
        self.subshell_layout.addWidget(self.ff_box)
        self.states_tabs.addTab(self.subshell_tab, "")
        self.subshell_box_dict = {"s": self.ss_box, "p": self.pp_box,
                                  "d": self.dd_box, "f": self.ff_box}
        self.subshell_box_list = self.subshell_box_dict.values()

        # ORBITAL TAB
        self.orbital_tab = QWidget()
        self.orbital_tab.setObjectName("orbital_tab")
        self.layoutWidget_2 = QWidget(self.orbital_tab)
        self.layoutWidget_2.setGeometry(QRect(10, 0, 181, 211))
        self.layoutWidget_2.setObjectName("layoutWidget_2")
        self.orbital_grid = QGridLayout(self.layoutWidget_2)
        self.orbital_grid.setContentsMargins(0, 0, 0, 0)
        self.orbital_grid.setObjectName("orbital_grid")
        self.f1_box = QCheckBox(self.layoutWidget_2)
        self.f1_box.setFont(self.font2)
        self.f1_box.setCursor(QCursor(Qt.CursorShape.ArrowCursor))
        self.f1_box.setCheckable(True)
        self.f1_box.setChecked(False)
        self.f1_box.setTristate(False)
        self.f1_box.setObjectName("f1_box")
        self.orbital_grid.addWidget(self.f1_box, 5, 1, 1, 1)
        self.dxy_box = QCheckBox(self.layoutWidget_2)
        self.dxy_box.setFont(self.font2)
        self.dxy_box.setCursor(QCursor(Qt.CursorShape.ArrowCursor))
        self.dxy_box.setCheckable(True)
        self.dxy_box.setChecked(False)
        self.dxy_box.setTristate(False)
        self.dxy_box.setObjectName("dxy_box")
        self.orbital_grid.addWidget(self.dxy_box, 2, 0, 1, 1)
        self.fn2_box = QCheckBox(self.layoutWidget_2)
        self.fn2_box.setFont(self.font2)
        self.fn2_box.setCursor(QCursor(Qt.CursorShape.ArrowCursor))
        self.fn2_box.setCheckable(True)
        self.fn2_box.setChecked(False)
        self.fn2_box.setTristate(False)
        self.fn2_box.setObjectName("fn2_box")
        self.orbital_grid.addWidget(self.fn2_box, 4, 1, 1, 1)
        self.fn1_box = QCheckBox(self.layoutWidget_2)
        self.fn1_box.setFont(self.font2)
        self.fn1_box.setCursor(QCursor(Qt.CursorShape.ArrowCursor))
        self.fn1_box.setCheckable(True)
        self.fn1_box.setChecked(False)
        self.fn1_box.setTristate(False)
        self.fn1_box.setObjectName("fn1_box")
        self.orbital_grid.addWidget(self.fn1_box, 4, 2, 1, 1)
        self.f0_box = QCheckBox(self.layoutWidget_2)
        self.f0_box.setFont(self.font2)
        self.f0_box.setCursor(QCursor(Qt.CursorShape.ArrowCursor))
        self.f0_box.setCheckable(True)
        self.f0_box.setChecked(False)
        self.f0_box.setTristate(False)
        self.f0_box.setObjectName("f0_box")
        self.orbital_grid.addWidget(self.f0_box, 5, 0, 1, 1)
        self.dyz_box = QCheckBox(self.layoutWidget_2)
        self.dyz_box.setFont(self.font2)
        self.dyz_box.setCursor(QCursor(Qt.CursorShape.ArrowCursor))
        self.dyz_box.setCheckable(True)
        self.dyz_box.setChecked(False)
        self.dyz_box.setTristate(False)
        self.dyz_box.setObjectName("dyz_box")
        self.orbital_grid.addWidget(self.dyz_box, 2, 1, 1, 1)
        self.dxz_box = QCheckBox(self.layoutWidget_2)
        self.dxz_box.setFont(self.font2)
        self.dxz_box.setCursor(QCursor(Qt.CursorShape.ArrowCursor))
        self.dxz_box.setCheckable(True)
        self.dxz_box.setChecked(False)
        self.dxz_box.setTristate(False)
        self.dxz_box.setObjectName("dxz_box")
        self.orbital_grid.addWidget(self.dxz_box, 3, 0, 1, 1)
        self.px_box = QCheckBox(self.layoutWidget_2)
        self.px_box.setFont(self.font2)
        self.px_box.setCursor(QCursor(Qt.CursorShape.ArrowCursor))
        self.px_box.setCheckable(True)
        self.px_box.setChecked(False)
        self.px_box.setTristate(False)
        self.px_box.setObjectName("px_box")
        self.orbital_grid.addWidget(self.px_box, 1, 2, 1, 1)
        self.dx2_box = QCheckBox(self.layoutWidget_2)
        self.dx2_box.setFont(self.font2)
        self.dx2_box.setCursor(QCursor(Qt.CursorShape.ArrowCursor))
        self.dx2_box.setCheckable(True)
        self.dx2_box.setChecked(False)
        self.dx2_box.setTristate(False)
        self.dx2_box.setObjectName("dx2_box")
        self.orbital_grid.addWidget(self.dx2_box, 3, 1, 1, 1)
        self.dz2_box = QCheckBox(self.layoutWidget_2)
        self.dz2_box.setFont(self.font2)
        self.dz2_box.setCursor(QCursor(Qt.CursorShape.ArrowCursor))
        self.dz2_box.setCheckable(True)
        self.dz2_box.setChecked(False)
        self.dz2_box.setTristate(False)
        self.dz2_box.setObjectName("dz2_box")
        self.orbital_grid.addWidget(self.dz2_box, 2, 2, 1, 1)
        self.fn3_box = QCheckBox(self.layoutWidget_2)
        self.fn3_box.setFont(self.font2)
        self.fn3_box.setCursor(QCursor(Qt.CursorShape.ArrowCursor))
        self.fn3_box.setCheckable(True)
        self.fn3_box.setChecked(False)
        self.fn3_box.setTristate(False)
        self.fn3_box.setObjectName("fn3_box")
        self.orbital_grid.addWidget(self.fn3_box, 4, 0, 1, 1)
        self.s_box = QCheckBox(self.layoutWidget_2)
        self.s_box.setFont(self.font2)
        self.s_box.setCursor(QCursor(Qt.CursorShape.ArrowCursor))
        self.s_box.setCheckable(True)
        self.s_box.setChecked(False)
        self.s_box.setTristate(False)
        self.s_box.setObjectName("s_box")
        self.orbital_grid.addWidget(self.s_box, 0, 0, 1, 1)
        self.py_box = QCheckBox(self.layoutWidget_2)
        self.py_box.setFont(self.font2)
        self.py_box.setCursor(QCursor(Qt.CursorShape.ArrowCursor))
        self.py_box.setCheckable(True)
        self.py_box.setChecked(False)
        self.py_box.setTristate(False)
        self.py_box.setObjectName("py_box")
        self.orbital_grid.addWidget(self.py_box, 1, 0, 1, 1)
        self.pz_box = QCheckBox(self.layoutWidget_2)
        self.pz_box.setFont(self.font2)
        self.pz_box.setCursor(QCursor(Qt.CursorShape.ArrowCursor))
        self.pz_box.setCheckable(True)
        self.pz_box.setChecked(False)
        self.pz_box.setTristate(False)
        self.pz_box.setObjectName("pz_box")
        self.orbital_grid.addWidget(self.pz_box, 1, 1, 1, 1)
        self.f2_box = QCheckBox(self.layoutWidget_2)
        self.f2_box.setFont(self.font2)
        self.f2_box.setCursor(QCursor(Qt.CursorShape.ArrowCursor))
        self.f2_box.setCheckable(True)
        self.f2_box.setChecked(False)
        self.f2_box.setTristate(False)
        self.f2_box.setObjectName("f2_box")
        self.orbital_grid.addWidget(self.f2_box, 5, 2, 1, 1)
        self.f3_box = QCheckBox(self.layoutWidget_2)
        self.f3_box.setFont(self.font2)
        self.f3_box.setCursor(QCursor(Qt.CursorShape.ArrowCursor))
        self.f3_box.setCheckable(True)
        self.f3_box.setChecked(False)
        self.f3_box.setTristate(False)
        self.f3_box.setObjectName("f3_box")
        self.orbital_grid.addWidget(self.f3_box, 6, 0, 1, 1)
        self.states_tabs.addTab(self.orbital_tab, "")
        self.orbital_box_dict = {"s": [self.s_box], "p": [self.py_box, self.pz_box, self.px_box],
                                  "d": [self.dxy_box, self.dyz_box, self.dz2_box, self.dxz_box, self.dx2_box],
                                 "f": [self.fn3_box, self.fn2_box, self.fn1_box, self.f0_box,
                                       self.f1_box, self.f2_box, self.f3_box]}
        self.orbital_box_list = [box for value in self.orbital_box_dict.values() for box in value]

        # LOAD STATUS LABEL
        self.load_label = QLabel(main_window)
        self.load_label.setGeometry(QRect(10, 110, 411, 20))
        self.load_label.setFont(self.font3)
        self.load_label.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.load_label.setObjectName("load_label")
        self.line1 = QFrame(main_window)
        self.line1.setGeometry(QRect(17, 130, 401, 20))
        self.line1.setFrameShape(QFrame.Shape.HLine)
        self.line1.setFrameShadow(QFrame.Shadow.Sunken)
        self.line1.setObjectName("line1")

        # PROPERTIES BOX
        self.properties_box = QGroupBox(main_window)
        self.properties_box.setGeometry(QRect(10, 320, 191, 91))
        self.properties_box.setFont(self.font1)
        self.properties_box.setObjectName("properties_box")
        self.layoutWidget2 = QWidget(self.properties_box)
        self.layoutWidget2.setGeometry(QRect(10, 30, 171, 61))
        self.layoutWidget2.setObjectName("layoutWidget2")
        self.properties_layout = QVBoxLayout(self.layoutWidget2)
        self.properties_layout.setContentsMargins(0, 0, 0, 0)
        self.properties_layout.setObjectName("properties_layout")
        self.name_text = QLineEdit(self.layoutWidget2)
        self.name_text.setFont(self.font2)
        self.name_text.setText("")
        self.name_text.setObjectName("name_text")
        self.properties_layout.addWidget(self.name_text)
        self.color_comb = QComboBox(self.layoutWidget2)
        self.color_comb.setObjectName("color_comb")
        self.color_comb.addItems(["", "red", "cyan", "green", "magenta", "blue", "yellow"])
        self.properties_layout.addWidget(self.color_comb)

        # DATASET LABEL
        self.line2 = QFrame(main_window)
        self.line2.setGeometry(QRect(10, 440, 411, 20))
        self.line2.setFrameShape(QFrame.Shape.HLine)
        self.line2.setFrameShadow(QFrame.Shadow.Sunken)
        self.line2.setObjectName("line2")
        self.dataset_label = QLabel(main_window)
        self.dataset_label.setGeometry(QRect(10, 420, 411, 20))
        self.dataset_label.setFont(self.font3)
        self.dataset_label.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.dataset_label.setObjectName("dataset_label")

        # DATASETS BOX
        self.datasets_box = QGroupBox(main_window)
        self.datasets_box.setGeometry(QRect(220, 455, 200, 173))
        self.datasets_box.setFont(self.font1)
        self.datasets_box.setObjectName("datasets_box")
        self.datasets_list = QListWidget(self.datasets_box)
        self.datasets_list.setGeometry(QRect(0, 19, 200, 155))
        self.datasets_list.setObjectName("datasets_list")

        # DATASET BUTTONS
        self.widget = QWidget(main_window)
        self.widget.setGeometry(QRect(10, 460, 191, 180))
        self.widget.setObjectName("widget")
        self.dataset_btn_layout = QVBoxLayout(self.widget)
        self.dataset_btn_layout.setContentsMargins(0, 0, 0, 0)
        self.dataset_btn_layout.setObjectName("dataset_btn_layout")
        self.add_data_btn = QPushButton(self.widget)
        self.add_data_btn.setObjectName("add_data_btn")
        self.add_data_btn.setFont(self.font3)
        self.dataset_btn_layout.addWidget(self.add_data_btn)
        self.add_total_btn = QPushButton(self.widget)
        self.add_total_btn.setObjectName("add_total_btn")
        self.add_total_btn.setFont(self.font3)
        self.dataset_btn_layout.addWidget(self.add_total_btn)
        self.remove_data_btn = QPushButton(self.widget)
        self.remove_data_btn.setObjectName("remove_data_btn")
        self.remove_data_btn.setFont(self.font3)
        self.dataset_btn_layout.addWidget(self.remove_data_btn)
        self.export_data_btn = QPushButton(self.widget)
        self.export_data_btn.setObjectName("export_data_btn")
        self.export_data_btn.setFont(self.font3)
        self.dataset_btn_layout.addWidget(self.export_data_btn)
        self.refresh_plot_btn = QPushButton(self.widget)
        self.refresh_plot_btn.setObjectName("refresh_plot_btn")
        self.refresh_plot_btn.setFont(self.font3)
        self.dataset_btn_layout.addWidget(self.refresh_plot_btn)

        self.dataset_btns = [self.add_data_btn, self.add_total_btn, self.remove_data_btn, self.export_data_btn, self.refresh_plot_btn]

        #PLOT SECTION
        self.plot_widget = QWidget(main_window)
        self.plot_widget.setGeometry(QRect(431, 10, 760, 630))
        self.plot_layout = QtWidgets.QVBoxLayout(self.plot_widget)
        self.canvas = FigureCanvas(Figure(figsize=(10, 10)))
        self.canvas.figure.set_tight_layout(True)
        self.plot_layout.addWidget(self.canvas)
        self.plot_layout.addWidget(NavigationToolbar(self.canvas, self))
        self.ax = self.canvas.figure.subplots()
        self.ax.set_xlabel('E [eV]', fontsize=20)
        self.ax.set_ylabel('DOS [states / eV]', fontsize=20)
        self.ax.tick_params(width=1, length=5, labelsize=20)

        self.retranslateUi(main_window)
        self.atom_tabs.setCurrentIndex(0)
        self.states_tabs.setCurrentIndex(0)
        QMetaObject.connectSlotsByName(main_window)

    def retranslateUi(self, main_window):
        _translate = QCoreApplication.translate
        main_window.setWindowTitle(_translate("main_window", "DosApp © AG"))
        self.spin_box.setTitle(_translate("main_window", "Spin:"))
        self.spin_both_btn.setText(_translate("main_window", "both"))
        self.spin_up_btn.setText(_translate("main_window", "up"))
        self.spin_down_btn.setText(_translate("main_window", "down"))
        self.load_box.setTitle(_translate("main_window", "Load VASP output"))
        self.load_txt.setPlaceholderText(_translate("main_window", "Select directory..."))
        self.browse_btn.setText(_translate("main_window", "Browse..."))
        self.load_btn.setText(_translate("main_window", "Load"))
        self.atom_sel_box.setTitle(_translate("main_window", "Atoms:"))
        self.atom_tabs.setTabText(self.atom_tabs.indexOf(self.atom_sel_tab), _translate("main_window", "Select"))
        self.atom_text.setPlaceholderText(_translate("main_window", "List atoms..."))
        self.atom_tabs.setTabText(self.atom_tabs.indexOf(self.atom_list_tab), _translate("main_window", "List"))
        self.states_box.setTitle(_translate("main_window", "States:"))
        self.ss_box.setText(_translate("main_window", "s"))
        self.pp_box.setText(_translate("main_window", "p"))
        self.dd_box.setText(_translate("main_window", "d"))
        self.ff_box.setText(_translate("main_window", "f"))
        self.states_tabs.setTabText(self.states_tabs.indexOf(self.subshell_tab), _translate("main_window", "Subshells"))
        self.f1_box.setText(_translate("main_window", "f(1)"))
        self.dxy_box.setText(_translate("main_window", "dxy"))
        self.fn2_box.setText(_translate("main_window", "f(-2)"))
        self.fn1_box.setText(_translate("main_window", "f(-1)"))
        self.f0_box.setText(_translate("main_window", "f(0)"))
        self.dyz_box.setText(_translate("main_window", "dyz"))
        self.dxz_box.setText(_translate("main_window", "dxz"))
        self.px_box.setText(_translate("main_window", "px"))
        self.dx2_box.setText(_translate("main_window", "dx2-y2"))
        self.dz2_box.setText(_translate("main_window", "dz2"))
        self.fn3_box.setText(_translate("main_window", "f(-3)"))
        self.s_box.setText(_translate("main_window", "s"))
        self.py_box.setText(_translate("main_window", "py"))
        self.pz_box.setText(_translate("main_window", "pz"))
        self.f2_box.setText(_translate("main_window", "f(2)"))
        self.f3_box.setText(_translate("main_window", "f(3)"))
        self.states_tabs.setTabText(self.states_tabs.indexOf(self.orbital_tab), _translate("main_window", "Orbitals"))
        self.load_label.setText(_translate("main_window", "Browse files and load a system"))
        self.properties_box.setTitle(_translate("main_window", "Properties"))
        self.name_text.setPlaceholderText(_translate("main_window", "Enter display name..."))
        self.datasets_box.setTitle(_translate("main_window", "Datasets"))
        self.dataset_label.setText(_translate("main_window", "Select atoms and states, and add them to datasets"))
        self.add_data_btn.setText(_translate("main_window", "Add dataset"))
        self.add_total_btn.setText(_translate("main_window", "Add total DOS"))
        self.remove_data_btn.setText(_translate("main_window", "Remove dataset"))
        self.export_data_btn.setText(_translate("main_window", "Export dataset"))
        self.refresh_plot_btn.setText(_translate("main_window", "Refresh plot"))



"""
Vasp Plot Suite
main script
"""


from PyQt6.QtWidgets import QApplication, QDialog
from sys import exit
from vaspplotsuite.mainwindow.mainwindow import MainWindow, buttonstyle
from vaspplotsuite.dos import DosAppView, DosAppController
from vaspplotsuite.bands import BandsAppView, BandsAppController


color1a = "#d90d1f"
color1d = "#D98088"
color2a = "#642870"
color2d = "#6C5870"


class MainView(QDialog, MainWindow):
    """
    Main window object. Details of the functioning of each of the modules
    can be found in their respective files and folders (dos and bands).
    """
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setupUi(self)
        self.DosButton.clicked.connect(self.open_dos_window)
        self.BandsButton.clicked.connect(self.open_bands_window)

    def open_dos_window(self):
        """
        Create and open new DosApp window
        """
        self.dos_window = DosAppView()
        DosAppController(self.dos_window)
        self.DosButton.setDisabled(True)
        self.DosButton.setStyleSheet(buttonstyle(color1d))
        self.dos_window.signal.closed.connect(self.close_dos_window)
        self.dos_window.show()

    def close_dos_window(self):
        self.DosButton.setEnabled(True)
        self.DosButton.setStyleSheet(buttonstyle(color1a))

    def open_bands_window(self):
        """
        Create and open new BandsApp window
        """
        self.bands_window = BandsAppView()
        BandsAppController(self.bands_window)
        self.BandsButton.setDisabled(True)
        self.BandsButton.setStyleSheet(buttonstyle(color2d))
        self.bands_window.signal.closed.connect(self.close_bands_window)
        self.bands_window.show()

    def close_bands_window(self):
        self.BandsButton.setEnabled(True)
        self.BandsButton.setStyleSheet(buttonstyle(color2a))


def main():
    """
    Main function - creates an instance of window (view)
    and applies controller to it
    """
    app = QApplication([])
    view = MainView()
    view.show()
    exit(app.exec())


if __name__ == "__main__":
    main()



import numpy as np
from scipy.integrate import trapz
from scipy.interpolate import interp1d
from lxml import etree


class RawDos:
    """
    Main DOS object, into which data is loaded from a directory and
    process to obtain: \n
    - array of total DOS \n
    - array of atomic DOS \n
    - spin-polarization (yes/no) and resolution (subshell/orbital) \n
    - Fermi energy \n
    Initialize as:
    data = RawDos(<path to vasprun.xml file>)
    In order to get atom and/or orbital resolved DOS, use "select" method as:
    selected = data([atoms], [states], spin(, name))
    """

    def __init__(self, path_to_file):
        """
        Constructor
        :param str path_to_file: address of vasprun.xml file
        """

        try:
            self.xml = etree.parse(path_to_file)
        except OSError:
            raise FileNotFoundError("vasprun.xml not present")

        self.name = self.xml.find("incar/i[@name='SYSTEM']").text.strip()

        # get atomic info
        self.atomdict, self.atomnames, self.atomnumbers = self._atom_info()

        # get electronic DOS info
        self.spin = None
        self.nedos, self.fermi = self._dos_info()
        self.totaldos, self.atomicdos, self.e = self._load_dos()
        self.e = self.e - self.fermi

        # get names of levels
        self.levels = self._load_levels()
        # check if subshell- or orbital-resolved
        if "px" in self.levels:
            self.lorbit = 11
        else:
            self.lorbit = 10

        # list subshells
        self.subshells = list(set(s[0] for s in self.levels) & {"s", "p", "d", "f"})

        # creating dictionaries of levels and their indices
        self.leveldict = {}
        for i in range(len(self.levels)):
            if self.spin:
                self.leveldict[self.levels[i]] = [2 * i, 2 * i + 1]
                self.spindict = {"up": [0], "down": [1], "both": [0, 1]}
            else:
                self.leveldict[self.levels[i]] = [i]
                self.spindict = None
        del self.xml

    def __str__(self):
        return f"DOS for {self.name} system"

    def get_total(self, spin="both"):
        """
        returns total DOS as array of energy and DOS \n
        :return: Dos
        """
        # selects the appropriate spin column
        if self.spin:
            if spin == "up":
                total = self.totaldos[:, 0]
            elif spin == "down":
                total = self.totaldos[:, 1]
            else:
                total = self.totaldos[:, [0, 1]].sum(1)
            name = f"{self.name}-total-{spin}"
        else:
            total = self.totaldos[:, 0]
            name = f"{self.name}-total"
        dos = np.column_stack([self.e, total])
        return Dos(dos, name)

    def integrate_total(self, lower, upper):
        """
        returns integrated total DOS \n
        somewhat reduntant, but the integrated values come from VASP,
        so we are keeping them \n
        :param float lower: lower bound
        :param float upper: upper bound
        :return: float
        """
        total = self.totaldos[:, -(1 + int(self.spin)):]
        if total.shape[-1] > 1:
            total = total.sum(1)
        inter = interp1d(self.e.flatten(), total.flatten())
        p1 = inter(lower)
        p2 = inter(upper)
        return p2 - p1

    def _atom_info(self):
        """
        read atomic info, used in constructor
        """

        atom_names = [a.text.strip() for a in self.xml.findall("atominfo/array[@name='atomtypes']/set/rc/c")][1::5]
        atom_numbers = [a.text.strip() for a in self.xml.findall("atominfo/array[@name='atomtypes']/set/rc/c")][0::5]
        atom_numbers = [int(n) for n in atom_numbers]
        atom_dict = {}
        cnt = 0
        for atom, number in zip(atom_names, atom_numbers):
            atom_dict[atom] = list(range(cnt, cnt + number))
            cnt = cnt + number
        return atom_dict, atom_names, atom_numbers

    def _dos_info(self):
        """
        read preliminary DOS info
        """
        nedos = int(self.xml.find("*/*/*[@name='NEDOS']").text)
        fermi = float(self.xml.find("*/*/*[@name='efermi']").text.strip())
        return nedos, fermi

    def _load_dos(self):
        """
        read actual DOS, used in constructor
        """
        totaldos = self.xml.findall("calculation/dos/total/array/set/set")
        if len(totaldos) == 2:
            self.spin = True
            totaldosup = self.xml.find("calculation/dos/total/array/set/set[@comment='spin 1']")
            totaldosup = np.array([i.text.split() for i in totaldosup]).astype(np.float32)
            totaldosdown = self.xml.find("calculation/dos/total/array/set/set[@comment='spin 2']")
            totaldosdown = np.array([i.text.split() for i in totaldosdown]).astype(np.float32)
            total_dos = np.column_stack([totaldosup[:, 1], totaldosdown[:, 1], totaldosup[:, 2], totaldosdown[:, 2]])
            e = totaldosup[:, 0]
            atomic = self.xml.findall("calculation/dos/partial/array/set/set/set")
            a_up = [[line.text.split() for line in atom] for atom in atomic[::2]]
            a_down = [[line.text.split() for line in atom] for atom in atomic[1::2]]
            a_up = np.array(a_up).astype(np.float32)
            a_down = np.array(a_down).astype(np.float32)
            zipped = list(
                zip([a_up[:, :, i] for i in range(a_up.shape[-1])], [a_down[:, :, i] for i in range(a_down.shape[-1])]))
            zipped = [i for x in zipped for i in x]
            atomic_dos = np.stack(zipped, axis=-1)[:, :, 2:]
        else:
            self.spin = False
            totaldos = np.array([i.text.split() for i in totaldos[0]]).astype(np.float32)
            total_dos = totaldos[:, 1:]
            e = totaldos[:, 0]
            atomic = self.xml.findall("calculation/dos/partial/array/set/set/set")
            a = [[line.text.split() for line in atom] for atom in atomic]
            atomic_dos = np.array(a).astype(np.float32)[:,:,1:]
        return total_dos, atomic_dos, e

    def _load_levels(self):
        """
        read list of levels, used in constructor
        """
        levels = [f.text.strip() for f in self.xml.findall("calculation/dos/partial/array/field")][1:]
        return np.array(levels)

    def select(self, atoms, states, spin="both", name=None):
        """
        selects atoms and/or states from total DOS \n
        :param iterable atoms: 1D array of selected atoms, by name or by number, matching atoms in raw dos
        :param iterable states: 1D array of selected states, matching levels in raw dos
        :param str spin: "up", "down" or "both"
        :param str name: name for the dataset
        :return: Dos object instance
        """

        selected = self.atomicdos
        if not type(atoms) in [np.ndarray, list]:
            raise TypeError("Invalid input type for atoms selection. List or 1D numpy array is acceptable.")
        if not type(states) in [np.ndarray, list]:
            raise TypeError("Invalid input type for states selection. List or 1D numpy array is acceptable.")
        if type(atoms) == np.ndarray:
            if len(atoms.shape) > 1:
                raise TypeError("Array of atoms has to be 1-dimensional.")
        if type(states) == np.ndarray:
            if len(states.shape) > 1:
                raise TypeError("Array of states has to be 1-dimensional.")
        atoms = np.array(atoms)
        states = np.array(states)

        if not (all(lev in self.levels for lev in states) or all(lev in self.subshells for lev in states)):
            raise ValueError("Some of the selected states are not present in the dataset.")

        if spin not in ["up", "down", "both"]:
            raise ValueError("Incorrect spin value selected. \"up\", \"down\" or \"both\" is accepted.")

        if name is None:
            name = f"[{' '.join(atoms.astype(str))}] - [{''.join(states.astype(str))}] - [{spin}]"

        # selecting atoms
        at_ind = []
        if np.issubdtype(atoms.dtype, str):
            if not all(at in self.atomdict.keys() for at in atoms):
                raise ValueError("Some of the selected atoms are not present in the dataset.")
            else:
                for atom in atoms:
                    at_ind += self.atomdict[atom]
        else:
            atoms = atoms - 1
            if not all(at in list(range(sum(self.atomnumbers))) for at in atoms):
                raise ValueError("Some of the selected atoms are not present in the dataset.")
            at_ind = [i for i in atoms]
        selected = selected[at_ind, :, :]

        # selecting states
        if (np.vectorize(len)(states) == 1).all():
            states = np.concatenate([np.argwhere(np.char.find(self.levels, c) != -1) for c in states]).flatten()
        if np.issubdtype(states.dtype, np.integer):
            states = self.levels[states]
        st_ind = []
        for s in states:
            st_ind += self.leveldict[s]
        if self.spin:
            spin_ind = self.spindict[spin]
            final_st_ind = []
            for spin in spin_ind:
                final_st_ind += st_ind[spin::2]
            selected = selected[:, :, final_st_ind]
        else:
            self.spin = None
            selected = selected[:, :, st_ind]

        # final processing
        selected = selected.sum(0).sum(1)
        selected = np.column_stack([self.e, selected])
        return Dos(selected, name)


class Dos:
    """
    Class for storing general Dos objects, selected from RawDos
    or loaded from a file.
    Data can be integrated using "integrate" method.
    The data can then be plotted using other scripts.
    """

    def __init__(self, dos, name):
        """
        :param np.ndarray dos: Array with E and eDOS values
        :param name: name for the dataset
        """
        self.dos = dos
        self.name = name

    def save(self, path):
        """
        saves eDOS to .csv file
        """
        np.savetxt(f"{path}/{self.name}.csv", self.dos, fmt='%.7f', delimiter=",")

    def integrate(self, lower, upper):
        """
        Integrates DOS in a given energy range
        :param float lower: lower bound for integration
        :param float upper: upper bound for integration
        :return: float
        """
        inter = interp1d(self.dos[:, 0], self.dos[:, 1])
        p1 = np.array([lower, inter(lower)]).reshape(1, 2)
        p2 = np.array([upper, inter(upper)]).reshape(1, 2)
        new = np.concatenate([self.dos, p1, p2])
        new = new[new[:, 0].argsort(), :]
        int_at_lower = trapz(new[new[:, 0] <= lower][:, 1], x=new[new[:, 0] <= lower][:, 0])
        int_at_upper = trapz(new[new[:, 0] <= upper][:, 1], x=new[new[:, 0] <= upper][:, 0])
        return int_at_upper - int_at_lower



from functools import partial
from PyQt6.QtWidgets import QFileDialog
from PyQt6.QtTest import QTest
from os import name as osname
from os import path as ospath
import numpy as np


class AppController:
    """
    Generic controller class for use in DosApp and BandsApp modules.
    """

    def __init__(self, view):
        self.view = view
        self.disable_window()
        osys = osname
        if osys == "posix":
            self.home_path = "/Users"
        else:
            self.home_path = "C:\\"
        # Connecting buttons to functions
        self.view.atom_tabs.currentChanged.connect(self.clear_atom_tabs)
        self.view.states_tabs.currentChanged.connect(self.clear_states_tabs)
        self.view.browse_btn.clicked.connect(partial(self.browse, self.view))
        self.view.load_btn.clicked.connect(self.load_data)
        self.view.add_data_btn.clicked.connect(self.add_dataset)
        self.view.remove_data_btn.clicked.connect(self.remove_dataset)

    def browse(self, view):
        """
        Browse directories
        """
        chosen_file = QFileDialog.getOpenFileName(view, "Choose file:", self.home_path, "(vasprun.xml)")[0]
        self.view.load_txt.setText(chosen_file)
        new_home_path = ospath.dirname(ospath.dirname(chosen_file))
        self.home_path = new_home_path

    def load_data(self):
        pass

    def add_dataset(self):
        """
        Generic method for loading data based on selection from the window.
        Details that differ between eDOS and bands are defined in plot_added_data within
        both controller subclasses
        """
        if self.view.atom_tabs.currentIndex() == 0:
            atoms = [self.view.atom_comb.currentText()]
        else:
            atoms = self.view.atom_text.text().split()
            try:
                atoms = np.array(atoms).astype(int)
            except ValueError:
                atoms = np.array(atoms)
        states = []
        if self.view.states_tabs.currentIndex() == 0:
            for box in self.view.subshell_box_list:
                if box.isChecked():
                    states.append(box.text())
        else:
            for box in self.view.orbital_box_list:
                if box.isChecked():
                    states.append(box.text())
        states = np.array(states)
        if len(states) > 0:
            states[np.where(states == "dx2-y2")] = "dx2"
        if self.view.spin_box.isEnabled() and self.view.spin_btn_group.checkedButton() is not None:
            spin = self.view.spin_btn_group.checkedButton().text()
        else:
            spin = "both"
        color = self.view.color_comb.currentText()
        name = self.view.name_text.text()
        condition = (len(states) > 0) and (len(atoms) > 0) and \
                    (not np.isin("", atoms)) and (name != "") and (color != "")
        if condition:
            try:
                self.plot_added_data(atoms, states, spin, name, color)
                self.view.dataset_label.setText("Dataset successfully added")
                self.reset_input()
            except Exception as e:
                self.view.dataset_label.setText(e.args[0])
        else:
            self.view.dataset_label.setText("Make sure you made a valid (non-null) selection")
        self.toggle_plot()
        self.populate_items()
        QTest.qWait(2000)
        self.view.dataset_label.setText("Select atoms and states, and add them to datasets")

    def plot_added_data(self, *args):
        pass

    def remove_dataset(self):
        pass

    def toggle_plot(self):
        pass

    def populate_items(self):
        pass

    def clear_atom_tabs(self):
        """
        Reset input in atom tabs
        """
        self.view.atom_comb.setCurrentIndex(0)
        self.view.atom_text.clear()

    def clear_states_tabs(self):
        """
        Reset input in states tabs
        """
        for box in self.view.subshell_box_list:
            box.setChecked(False)
        for box in self.view.orbital_box_list:
            box.setChecked(False)

    def reset_input(self):
        """
        Reset selection in the window
        """
        self.clear_atom_tabs()
        self.clear_states_tabs()
        self.view.name_text.clear()
        self.view.color_comb.setCurrentIndex(0)
        self.view.spin_btn_group.setExclusive(False)
        for btn in self.view.spin_btn_list:
            btn.setChecked(False)
        self.view.spin_btn_group.setExclusive(True)

    def disable_window(self):
        """
        Disable functions in the window
        """
        self.view.atom_sel_box.setDisabled(True)
        self.reset_input()
        self.view.states_box.setDisabled(True)
        self.view.spin_box.setDisabled(True)
        for btn in self.view.dataset_btns:
            btn.setDisabled(True)
        self.view.datasets_list.clear()
        self.view.datasets_list.setDisabled(True)
        for box in self.view.subshell_box_list:
            box.setDisabled(True)
        for box in self.view.orbital_box_list:
            box.setDisabled(True)
        self.view.properties_box.setDisabled(True)

    def adjust_window(self):
        """
        Selectively enabling functionalities in the window
        based on the loaded VASP data
        """
        self.disable_window()
        # enable and load atoms
        self.view.atom_sel_box.setEnabled(True)
        self.view.atom_comb.clear()
        self.view.atom_comb.addItems([""] + self.loaded_data.atomnames)
        # enable spin if spin-polarized
        if self.loaded_data.spin:
            self.view.spin_box.setEnabled(True)
        else:
            self.view.spin_box.setDisabled(True)
        # enable states and subshells
        self.view.states_box.setEnabled(True)
        self.view.subshell_tab.setEnabled(True)
        for level in self.loaded_data.subshells:
            self.view.subshell_box_dict[level].setEnabled(True)
        # enable orbitals if lorbit is 11
        if self.loaded_data.lorbit == 10:
            self.view.orbital_tab.setDisabled(True)
        else:
            self.view.orbital_tab.setEnabled(True)
            for level in self.loaded_data.subshells:
                for box in self.view.orbital_box_dict[level]:
                    box.setEnabled(True)
        # enable processing buttons
        for btn in self.view.dataset_btns:
            btn.setEnabled(True)
        self.view.datasets_list.setEnabled(True)
        self.view.datasets_list.clear()
        self.view.properties_box.setEnabled(True)



"""
VASP-DOS-tools - library for processing electronic density of states (eDOS) from VASP calculations \n
Reads data from vasprun.xml files
AG, 2022 \n
"""


import numpy as np
from vaspplotsuite.dos.vaspdostools.dostypes import RawDos, Dos


def extract(directory):
    """
    Reads DOS from VASP output \n
    See DOS class for more information \n
    :param directory:
    :return: DOS
    """
    return RawDos(directory)


def open_file(path, name=None):
    """
    Reads DOS from previously saved .csv \n
    :param str path: location
    :param str name: name for the system
    :return: Dos
    """
    dos_array = np.genfromtxt(path, delimiter=',', dtype=np.float64)
    if name is None:
        name = path.split(".")[-1]
    return Dos(dos_array, name)



from sys import exit
from os import path
from os import name as oname
from PyQt6.QtWidgets import QSizePolicy, QPushButton, QApplication, QDialog, QLabel, QFrame, QWidget
from PyQt6.QtGui import QFont
from PyQt6.QtCore import QMetaObject, QCoreApplication, QRect, QSize
from PyQt6.QtCore import Qt
from vaspplotsuite.static import font


def buttonstyle(color):
    return f"background-color: {color}; color: white; border-radius: 5px; " \
           f"border: solid; border-width: 1px;"


class MainWindow(object):

    if oname == "nt":
        q = 0.75
    else:
        q = 1
    font1 = QFont(font[0])
    font1.setPointSize(int(36*q))
    font2 = QFont(font[0])
    font2.setPointSize(int(24*q))
    font3 = QFont(font[0])
    font3.setPointSize(int(20*q))
    font4 = QFont(font[0])
    font4.setPointSize(int(12*q))

    def setupUi(self, Window):
        Window.setObjectName("Window")
        Window.resize(600, 415)
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Window.sizePolicy().hasHeightForWidth())
        Window.setSizePolicy(sizePolicy)
        Window.setMinimumSize(QSize(600, 415))
        Window.setMaximumSize(QSize(600, 415))
        Window.setStyleSheet("")
        self.DosButton = QPushButton(Window)
        self.DosButton.setGeometry(QRect(30, 220, 240, 81))
        self.DosButton.setFont(self.font1)
        self.DosButton.setAutoFillBackground(False)
        self.DosButton.setStyleSheet(buttonstyle("#d90d1f"))
        self.DosButton.setIconSize(QSize(20, 20))
        self.DosButton.setObjectName("DosButton")
        self.BandsButton = QPushButton(Window)
        self.BandsButton.setGeometry(QRect(330, 220, 240, 81))
        self.BandsButton.setFont(self.font1)
        self.BandsButton.setStyleSheet(buttonstyle("#642870"))
        self.BandsButton.setObjectName("BandsButton")
        self.label = QLabel(Window)
        self.label.setGeometry(QRect(25, 20, 550, 30))
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        self.label.setFont(self.font2)
        self.label.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.label.setObjectName("label")
        self.label_2 = QLabel(Window)
        self.label_2.setGeometry(QRect(10, 170, 580, 25))
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_2.sizePolicy().hasHeightForWidth())
        self.label_2.setSizePolicy(sizePolicy)
        self.label_2.setFont(self.font3)
        self.label_2.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.label_2.setObjectName("label_2")
        self.label_3 = QLabel(Window)
        self.label_3.setGeometry(QRect(100, 340, 400, 30))
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_3.sizePolicy().hasHeightForWidth())
        self.label_3.setSizePolicy(sizePolicy)
        self.label_3.setFont(self.font4)
        self.label_3.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.label_3.setObjectName("label_3")
        self.label_4 = QLabel(Window)
        self.label_4.setGeometry(QRect(100, 380, 400, 30))
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_4.sizePolicy().hasHeightForWidth())
        self.label_4.setSizePolicy(sizePolicy)
        self.label_4.setFont(self.font4)
        self.label_4.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.label_4.setObjectName("label_4")
        self.label_5 = QLabel(Window)
        self.label_5.setGeometry(QRect(100, 360, 400, 30))
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_5.sizePolicy().hasHeightForWidth())
        self.label_5.setFont(self.font4)
        self.label_5.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.label_5.setObjectName("label_5")
        self.line = QFrame(Window)
        self.line.setGeometry(QRect(23, 320, 559, 20))
        self.line.setFrameShape(QFrame.Shape.HLine)
        self.line.setFrameShadow(QFrame.Shadow.Sunken)
        self.line.setObjectName("line")
        self.widget = QWidget(Window)
        self.widget.setGeometry(QRect(255, 75, 92, 80))
        path_to_img = path.join(path.dirname(__file__), "app-logo.png")
        self.widget.setStyleSheet(f"background-image: url({path_to_img});")
        print(self.widget.styleSheet())
        self.widget.setObjectName("widget")

        self.retranslateUi(Window)
        QMetaObject.connectSlotsByName(Window)

    def retranslateUi(self, Window):
        _translate = QCoreApplication.translate
        Window.setWindowTitle(_translate("Window", "VASP Plot Suite © AG"))
        self.DosButton.setText(_translate("Window", "eDOS"))
        self.BandsButton.setText(_translate("Window", "Bands"))
        self.label.setText(_translate("Window", "Welcome to VASP Plot Suite!"))
        self.label_2.setText(_translate("Window", "What kind of output data would you like to analyze?"))
        self.label_3.setText(_translate("Window", "© Adam Grzelak"))
        self.label_4.setText(_translate("Window", "For questions and support: contact@adamgrzelak.com"))
        self.label_5.setText(_translate("Window", "www.adamgrzelak.com"))


if __name__ == "__main__":
    app = QApplication([])
    Window = QDialog()
    ui = MainWindow()
    ui.setupUi(Window)
    Window.show()
    exit(app.exec())



"""
Application for plotting electronic band structure graphs
from VASP calculation output
Frontend based on PyQt6
Backend is handled by VASP-Band-tools mini-library
Developed by AG
(C) 2022
"""

from sys import exit
from PyQt6.QtWidgets import QApplication, QDialog
from PyQt6.QtTest import QTest
from PyQt6.QtCore import pyqtSignal, QObject
from vaspplotsuite.bands.qtdesign import BandsAppWindow
from vaspplotsuite.bands.vaspbandtools import BandStructure
from vaspplotsuite.mainwindow.controller import AppController
import numpy as np


class Signal(QObject):
    closed = pyqtSignal()


class BandsAppView(QDialog, BandsAppWindow):
    """
    Main window object for BandsApp
    """
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setupUi(self)
        self.signal = Signal()

    def closeEvent(self, e):
        self.signal.closed.emit()


class BandsAppController(AppController):

    def __init__(self, view):
        super().__init__(view)
        self.view.refresh_plot_btn.clicked.connect(self.refresh_plot)

    def load_data(self):
        """
        Load VASP output into a Bandstructure object and plot the simple bands
        """
        QTest.qWait(5)
        self.view.load_label.setText("Loading...")
        QTest.qWait(5)
        self.active_path = self.view.load_txt.text()
        try:
            QTest.qWait(5)
            self.view.load_label.setText("Loading bands...")
            QTest.qWait(5)
            self.loaded_data = BandStructure(self.active_path, load_all=False)
            data_per_point = self.loaded_data.nbands * self.loaded_data.nions
            if self.loaded_data.spin:
                data_per_point = data_per_point * 2
            gen = self.loaded_data._datagen()
            data = []
            i = 0
            for r in gen:
                i += 1
                if i % data_per_point == 0:
                    QTest.qWait(5)
                    self.view.load_label.setText(f"Processing k-point {i // data_per_point}/{self.loaded_data.nkpts}...")
                    QTest.qWait(5)
                data.append(r)
            QTest.qWait(5)
            self.view.load_label.setText("Finishing up...")
            QTest.qWait(5)
            self.loaded_data._set_data(data)
            # account for different shorthands for dx2-y2 orbital depending on VASP version
            if "x2-y2" in self.loaded_data.levels:
                self.loaded_data.levels[np.where(self.loaded_data.levels == "x2-y2")] = "dx2"
                self.loaded_data.leveldict["dx2"] = self.loaded_data.leveldict.pop("x2-y2")
            while self.view.ax._children:
                self.view.ax._children[0].remove()
            self.adjust_window()
            QTest.qWait(5)
            self.view.load_label.setText("Plotting...")
            QTest.qWait(5)
            self.plot_basic_bands()
            self.toggle_plot()
            msg = f"<b>{self.loaded_data.name}</b> system was loaded successfully"
            self.view.load_label.setText(msg)
        except Exception as e:
            self.view.load_label.setText(e.args[0])
            QTest.qWait(2000)
            self.view.load_label.setText("Browse files and load a system")
            self.disable_window()

    def plot_added_data(self, atoms, states, spin, name, color):
        """
        Plot selected resolved data
        """
        sel_dataset = self.loaded_data.select(atoms, states, spin, name)
        if sel_dataset.data.max() != 0:
            markers = sel_dataset.data / sel_dataset.data.max() * 100
            markers = markers.flatten("F")
        else:
            markers = np.zeros(self.loaded_data.nkpts*self.loaded_data.nbands)
        new_xaxis = np.concatenate([self.loaded_data.xaxis]*self.loaded_data.nbands)
        flattened_bands = self.loaded_data.bands[0,:,:].flatten("F")
        self.view.ax.scatter(new_xaxis, flattened_bands,
                             s=markers, color=color, label=sel_dataset.name)

    def remove_dataset(self):
        """
        Remove a dataset from dataset list
        """
        try:
            to_remove = self.view.datasets_list.currentItem().text()
            for line in self.view.ax._children:
                if line._label == to_remove:
                    line.remove()
        except Exception:
            self.view.dataset_label.setText("Select a dataset to remove")
            QTest.qWait(2000)
            self.view.dataset_label.setText("Select atoms and states, and add them to datasets")
        self.populate_items()
        self.toggle_plot()

    def populate_items(self):
        """
        Refresh content of dataset list
        """
        self.view.datasets_list.clear()
        labels = []
        for line in self.view.ax._children:
            if "_" not in line._label:
                labels.append(line._label)
        labels = list(set(labels))
        for label in labels:
            self.view.datasets_list.addItem(f"{label}")

    def plot_basic_bands(self):
        """
        Plot simple band structure as the background for further selection
        """
        if self.view.ax.lines:
            low, high = self.view.ax.get_ylim()
        else:
            low, high = self.loaded_data.bands.min() - 1, self.loaded_data.bands.max() + 1
        for i in range(self.loaded_data.nbands):
            self.view.ax.plot(self.loaded_data.xaxis, self.loaded_data.bands[0, :, i], color="black", label="_nolegend_")
        self.view.ax.hlines(0, 0, self.loaded_data.xaxis[-1], color="black", ls="--")
        vlines = self.loaded_data.xaxis[np.where(self.loaded_data.xaxis[1:] - self.loaded_data.xaxis[:-1] == 0)]
        vlines = np.insert(vlines, 0, 0)
        vlines = np.append(vlines, self.loaded_data.xaxis.max())
        self.view.ax.set_ylim([low, high])
        self.view.ax.vlines(vlines, low, high, color="black", ls="dotted")
        self.view.ax.set_xlim([self.loaded_data.xaxis.min(), self.loaded_data.xaxis.max()])
        self.view.ax.set_xticks(vlines)
        self.view.ax.set_xticklabels([])
        self.view.canvas.draw()

    def toggle_plot(self):
        """
        Refresh plot area
        """
        if self.view.ax.get_legend():
            self.view.ax.get_legend().remove()
        if len(self.view.ax._children) > self.loaded_data.nbands+2:
            self.legend = self.view.ax.legend(prop={'size': 15})
        self.view.canvas.draw()

    def refresh_plot(self):
        """
        Refresh plot area and update k-points list
        """
        self.add_kpoints()
        self.toggle_plot()

    def add_kpoints(self):
        """
        Add tick names to k axis
        """
        kpoints = self.view.kpoint_text.text().upper().replace(",", " ").replace("G", "Γ").split()
        if len(kpoints) == len(self.view.ax.get_xticks()):
            self.view.ax.set_xticklabels(kpoints)
        else:
            self.view.dataset_label.setText("Invalid k-point list")
            QTest.qWait(2000)
            self.view.dataset_label.setText("Add datasets by selecting atoms and states, or plot datasets")


def main():
    """
    Main function - creates an instance of window (view)
    and applies controller to it
    """
    app = QApplication([])
    view = BandsAppView()
    BandsAppController(view)
    view.show()
    exit(app.exec())


if __name__ == "__main__":
    main()



from PyQt6.QtGui import QFontDatabase, QGuiApplication
from os import path

a = QGuiApplication([])
font_path = path.dirname(__file__)
font_path = path.join(font_path, "Montserrat-Regular.ttf")
font = QFontDatabase.addApplicationFont(font_path)
font = QFontDatabase.applicationFontFamilies(font)



import numpy as np
from lxml import etree


class SimpleBands:
    """
    SimpleBands class for reading band structure (not resolved for ions and levels).
    Initialize as:
    data = SimpleBands(<path to your vasprun.xml file>)
    This class is not intended to be used separately; all of its features and more
    are implemented in the child BandStructure class.
    """

    def __init__(self, path_to_file):
        """
        Constructor
        :param str path_to_file: address of vasprun.xml file
        """
        try:
            self.xml = etree.parse(path_to_file)
        except OSError:
            raise FileNotFoundError("vasprun.xml not present")

        self.name = self.xml.find("incar/i[@name='SYSTEM']").text.strip()
        self.efermi, self.nions, self.nkpts, self.nbands, self.levels, self.subshells, self.lorbit = self._read_info()
        self.atomdict, self.atomnames, self.atomnumbers = self._read_atom_info()
        self.kpoints = self._get_kpoints()
        self.lattice, self.rec_lattice = self._get_dims()
        self.bands = self._get_data()
        self.xaxis = self._get_xaxis()

    def _read_info(self):
        """
        Read information about the system: Fermi energy, number of atoms, 
        number of k-points, number of bands, and list of levels
        """
        efermi = float(self.xml.find("*/*/*[@name='efermi']").text)
        nions = int(self.xml.find("atominfo/atoms").text)
        nkpts = int(len(self.xml.find("kpoints/varray[@name='kpointlist']")))
        nbands = int(self.xml.find("*/*/*[@name='NBANDS']").text)
        levels = np.array([f.text.strip() for f in self.xml.findall("calculation/dos/partial/array/field")][1:])
        if "x2-y2" in levels:
            levels[np.where(levels == "x2-y2")] = "dx2"
        subshells = list(set(s[0] for s in levels) & {"s", "p", "d", "f"})
        if (np.array(list(map(lambda x: len(x), levels))) > 1).any():
            lorbit = 11
        else:
            lorbit = 10
        return efermi, nions, nkpts, nbands, levels, subshells, lorbit
    
    def _read_atom_info(self):
        """
        Read atomic info - atom names and their number - and put them into a dictionary
        """
        atom_names = [a.text.strip() for a in self.xml.findall("atominfo/array[@name='atomtypes']/set/rc/c")][1::5]
        atom_numbers = [a.text.strip() for a in self.xml.findall("atominfo/array[@name='atomtypes']/set/rc/c")][0::5]
        atom_numbers = [int(n) for n in atom_numbers]
        atom_dict = {}
        cnt = 0
        for atom, number in zip(atom_names, atom_numbers):
            atom_dict[atom] = list(range(cnt, cnt + number))
            cnt = cnt + number
        return atom_dict, atom_names, atom_numbers

    def _get_dims(self):
        """
        Get matrices of direct and reciprocal lattice
        """
        lattice = [line.text.split() for line
                   in self.xml.findall("structure[@name='initialpos']/crystal/varray[@name='basis']/v")]
        lattice = np.array(lattice).astype(np.float32)
        rec_lattice = [line.text.split() for line
                       in self.xml.findall("structure[@name='initialpos']/crystal/varray[@name='rec_basis']/v")]
        rec_lattice = np.array(rec_lattice).astype(np.float32)
        return lattice, rec_lattice

    def _get_kpoints(self):
        """
        Get an array of kpoints
        """
        kpoints = np.array([kpoint.text.split() for kpoint
                            in self.xml.findall("kpoints/varray[@name='kpointlist']/v")]).astype(np.float32)
        return kpoints

    def _get_data(self):
        """
        Read eigenvals
        """
        band_sets = self.xml.findall("calculation/projected/eigenvalues/array/set/set/set/r")
        data = np.array([r.text.split()[0] for r in band_sets]).astype(np.float32)
        data = data.reshape(-1, self.nbands)
        data = data - self.efermi
        if data.shape[0] == self.nkpts:
            self.spin = False
            return data
        elif data.shape[0] == 2 * self.nkpts:
            self.spin = True
            data = np.stack([data[:self.nkpts, :], data[self.nkpts:, :]])
            return data
        else:
            raise TypeError("Something went wrong. Is the data set spin-polarized or not?")

    def _get_xaxis(self):
        """
        Process the k-points and reciprocal lattice arrays in order to get
        the x-axis for plotting (with accurate distance in BZ)
        """
        rel_coords = np.dot(self.kpoints, self.rec_lattice)
        dists = rel_coords[1:] - rel_coords[:-1]
        dists = (dists ** 2).sum(1) ** (1 / 2)
        axis = dists.cumsum()
        axis = np.insert(axis, 0, 0)
        return axis



"""
Application for plotting electronic density of states (eDOS) graphs
from VASP calculation output
Frontend based on PyQt6
Backend is handled by VASP-DOS-tools mini-library
Developed by AG
(C) 2022
"""

from sys import exit
from PyQt6.QtWidgets import QApplication, QDialog
from PyQt6.QtTest import QTest
from PyQt6.QtCore import pyqtSignal, QObject
from vaspplotsuite.dos.qtdesign import DosAppWindow
import vaspplotsuite.dos.vaspdostools as vdt
from vaspplotsuite.mainwindow.controller import AppController
from os import path as ospath
import numpy as np


class Signal(QObject):
    closed = pyqtSignal()


class DosAppView(QDialog, DosAppWindow):
    """
    Main window object for DosApp
    """
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setupUi(self)
        self.signal = Signal()

    def closeEvent(self, e):
        self.signal.closed.emit()


class DosAppController(AppController):

    def __init__(self, view):
        super().__init__(view)
        # Connecting buttons to functions
        self.view.add_total_btn.clicked.connect(self.add_total_dos)
        self.view.export_data_btn.clicked.connect(self.export_dataset)
        self.view.refresh_plot_btn.clicked.connect(self.toggle_plot)

    def load_data(self):
        """
        Load VASP output into a Dos object
        """
        for line in self.view.ax.lines:
            self.view.ax.lines.remove(line)
        self.toggle_plot()
        QTest.qWait(5)
        self.view.load_label.setText("Loading...")
        QTest.qWait(5)
        self.active_path = self.view.load_txt.text()
        try:
            path_to_file = self.view.load_txt.text()
            self.loaded_data = vdt.extract(path_to_file)
            # account for different shorthands for dx2-y2 orbital depending on VASP version
            if "x2-y2" in self.loaded_data.levels:
                self.loaded_data.levels[np.where(self.loaded_data.levels == "x2-y2")] = "dx2"
                self.loaded_data.leveldict["dx2"] = self.loaded_data.leveldict.pop("x2-y2")
            QTest.qWait(5)
            msg = f"<b>{self.loaded_data.name}</b> system was loaded successfully"
            QTest.qWait(5)
            self.view.load_label.setText(msg)
            self.adjust_window()
        except Exception as e:
            self.view.load_label.setText(e.args[0])
            QTest.qWait(2000)
            self.view.load_label.setText("Browse files and load a system")
            self.disable_window()

    def plot_added_data(self, atoms, states, spin, name, color):
        """
        Plot selected resolved data
        """
        sel_dataset = self.loaded_data.select(atoms, states, spin, name)
        self.view.ax.plot(sel_dataset.dos[:, 0], sel_dataset.dos[:, 1],
                          color=color, label=sel_dataset.name,
                          linewidth=4)

    def add_total_dos(self):
        """
        Select total DOS from the Dos object and
        add it to dataset list
        """
        if self.view.spin_box.isEnabled() and self.view.spin_btn_group.checkedButton() is not None:
            spin = self.view.spin_btn_group.checkedButton().text()
        else:
            spin = "both"
        color = self.view.color_comb.currentText()
        if color == "":
            color = "black"
        sel_dataset = self.loaded_data.get_total(spin)
        self.view.ax.plot(sel_dataset.dos[:, 0], sel_dataset.dos[:, 1],
                          color=color, label=sel_dataset.name,
                          linewidth=4)
        self.view.dataset_label.setText("Dataset successfully added")
        self.reset_input()
        self.populate_items()
        self.toggle_plot()
        QTest.qWait(2000)
        self.view.dataset_label.setText("Select atoms and states, and add them to datasets")

    def populate_items(self):
        """
        Refresh content of dataset list
        """
        self.view.datasets_list.clear()
        for line in self.view.ax.lines:
            self.view.datasets_list.addItem(f"{line._label}")

    def remove_dataset(self):
        """
        Remove a dataset from dataset list
        """
        if self.view.ax.lines:
            self.view.ax.lines.remove(self.view.ax.lines[self.view.datasets_list.currentRow()])
            self.populate_items()
        self.toggle_plot()

    def export_dataset(self):
        """
        Save the selected dataset to .csv file
        """
        if len(self.view.ax.lines) > 0:
            to_save = self.view.ax.lines[self.view.datasets_list.currentRow()]
            path_to_save = ospath.dirname(self.active_path)
            np.savetxt(f"{path_to_save}/{to_save._label}.csv", to_save._xy, fmt='%.7f', delimiter=",")

    def toggle_plot(self):
        """
        Refresh plot area
        """
        try:
            if len(self.view.ax.lines) > 0:
                low, high = self.view.ax.get_xlim()
                ymax = max([(line._y[np.where((line._x >= low) & (line._x <= high))]).max() for line in self.view.ax.lines]) * 1.1
                self.view.ax.set_ylim(0, ymax)
                self.view.ax.vlines(0, 0, ymax, colors='gray', linestyles='dashed', linewidth=4)
                self.legend = self.view.ax.legend(prop={'size': 15})
            else:
                if self.view.ax.get_legend():
                    self.view.ax.get_legend().remove()
        except Exception as e:
            self.view.dataset_label.setText(e.args[0])
            QTest.qWait(2000)
            self.view.dataset_label.setText("Add datasets by selecting atoms and states, or plot datasets")
        self.view.canvas.draw()
        self.populate_items()


def main():
    """
    Main function - creates an instance of window (view)
    and applies controller to it
    """
    app = QApplication([])
    view = DosAppView()
    DosAppController(view)
    view.show()
    exit(app.exec())


if __name__ == "__main__":
    main()



import numpy as np
from vaspplotsuite.bands.vaspbandtools.simplebands import SimpleBands


class BandStructure(SimpleBands):
    """
    BandStructure for reading orbital- and ion-resolved data.
    Inherits from SimpleBands and adds data selection funcionality.
    Initialize as:
    data = BandStructure(<path to your vasprun.xml file>)
    If the data is spin-polarized, the array of projected bands is of the shape:
    (2 (up and down spin), number of kpoints, number of bands, number of atoms/ions, number of levels (subshells or orbitals))
    If it is not spin-polarized, the shape is:
    (number of kpoints, number of bands, number of atoms/ions, number of levels (subshells or orbitals))
    """

    def __init__(self, path, load_all=True):
        """
        Constructor
        :param str path_to_file: address of vasprun.xml file
        """
        super().__init__(path)
        self.leveldict, self.spindict = self._get_level_dict()
        if load_all:
            self._get_proj_bands_verbose()

    def select(self, atoms, states, spin="both", name=None):
        """
        selects atoms and/or states for band projection \n
        :param iterable atoms: 1D array of selected atoms, by name or by number, matching atoms in simplebands
        :param iterable states: 1D array of selected states, matching levels in simplebands
        :param str spin: "up", "down" or "both"
        :param str name: name for the dataset
        :return: ProjectedBands object, where data is of the shape (number of kpoints, number of bands)
        """

        selected = self.proj_bands
        if not type(atoms) in [np.ndarray, list]:
            raise TypeError("Invalid input type for atoms selection. List or 1D numpy array is acceptable.")
        if not type(states) in [np.ndarray, list]:
            raise TypeError("Invalid input type for states selection. List or 1D numpy array is acceptable.")
        if type(atoms) == np.ndarray:
            if len(atoms.shape) > 1:
                raise TypeError("Array of atoms has to be 1-dimensional.")
        if type(states) == np.ndarray:
            if len(states.shape) > 1:
                raise TypeError("Array of states has to be 1-dimensional.")
        atoms = np.array(atoms)
        states = np.array(states)

        if not (all(lev in self.levels for lev in states) or all(lev in self.subshells for lev in states)):
            raise ValueError("Some of the selected states are not present in the dataset.")

        if spin not in ["up", "down", "both"]:
            raise ValueError("Incorrect spin value selected. \"up\", \"down\" or \"both\" is accepted.")

        if name is None:
            name = f"[{' '.join(atoms.astype(str))}] - [{''.join(states.astype(str))}] - [{spin}]"

        # selecting atoms
        at_ind = []
        if np.issubdtype(atoms.dtype, str):
            if not all(at in self.atomdict.keys() for at in atoms):
                raise ValueError("Some of the selected atoms are not present in the dataset.")
            else:
                for atom in atoms:
                    at_ind += self.atomdict[atom]
        else:
            atoms = atoms - 1
            if not all(at in list(range(sum(self.atomnumbers))) for at in atoms):
                raise ValueError("Some of the selected atoms are not present in the dataset.")
            at_ind = [i for i in atoms]

        # selecting states
        if (np.vectorize(len)(states) == 1).all():
            states = np.concatenate([np.argwhere(np.char.find(self.levels, c) != -1) for c in states]).flatten()
        if np.issubdtype(states.dtype, np.integer):
            states = self.levels[states]
        st_ind = []
        for s in states:
            st_ind += self.leveldict[s]

        # selecting spin and finalizing
        if self.spin:
            spin_ind = self.spindict[spin]
            selected = selected[:,:,:,at_ind,:][:,:,:,:,st_ind][spin_ind,:,:,:,:]
            selected = selected.sum(0).sum(-1).sum(-1)
        else:
            self.spin = None
            selected = selected[:,:,at_ind,:][:,:,:,st_ind]
            selected = selected.sum(-1).sum(-1)

        return ProjectedBands(selected, name)
        
    def _get_level_dict(self):
        """
        Create dictionary of levels and spins for selection
        """
        leveldict = {}
        for i in range(len(self.levels)):
            leveldict[self.levels[i]] = [i]
            if self.spin:
                spindict = {"up": [0], "down": [1], "both": [0, 1]}
            else:
                spindict = None
        return leveldict, spindict

    def _get_proj_bands(self):
        """
        Read orbital- and ion-resolved data in the background.
        """
        data = list(self._datagen())
        self._set_data(data)
    
    def _get_proj_bands_verbose(self):
        """
        Read orbital- and ion-resolved data verbosely.
        """
        data_per_point = self.nbands * 2 * self.nions
        gen = self._datagen()
        data = []
        i = 0
        for r in gen:
            i += 1
            if i % data_per_point == 0:
                print(f"Processed {i // data_per_point}/{self.nkpts} k-points...", end="\r")
            data.append(r)
        print()
        print("Finishing up...")
        self._set_data(data)
        print("Done.")

    def _datagen(self):
        """
        Create generator object with data.
        """
        data = self.xml.findall("calculation/projected/array/set/set/set/set/r")
        for r in data:
            yield np.array(r.text.split()).astype(np.float32)

    def _set_data(self, data):
        """
        Assign data created with a verbose progress-tracking loop.
        """
        proj = np.vstack(data)
        if self.spin:
            proj = proj.reshape(2, self.nkpts, self.nbands, self.nions, -1)
        else:
            proj = proj.reshape(1, self.nkpts, self.nbands, self.nions, -1)
        self.proj_bands = proj


class ProjectedBands:
    """
    Object for storing projected bands after selecting them.
    Consists simply of the data array and of the designated name.
    """
    def __init__(self, data, name):
        self.data = data
        self.name = name



from setuptools import setup, find_packages

description = """# VASP Plot Suite
### (C) AG 2022

Vasp Plot Suite is a program for processing and plotting electronic structure data
from VASP calculations output. It has two modules:

a) DosApp: for plotting electronic density of states (eDOS),<br>
b) BandsApp: for plotting electronic band structure.

Instructions for each of module can be found in their respective README files.

This library is available via pip:
<code>pip install vaspplotsuite</code>

After that, you can run it from your command line by typing <code>vaspplot</code>.

### Executables
This program can be compliled into an executable.
I can provide ones for Windows or MacOS upon request.

### This program is still being tested
If you notice any errors or discrepancies, or if you have any practical suggestions,
I would be grateful if you report them.
I have also included sample data for AgF2 system if you want to give it a try.

### Contact
contact@adamgrzelak.com

### Update history
15.05.2022 - updated fonts<br>
13.05.2022 - added command line script<br>
12.05.2022 - publication of version 1.0 Vasp Plot Suite as a whole on PyPI<br>
11.05.2022 - publication of functional BandsApp<br>
10.05.2022 - incorporated <code>matplotlib</code> widget into the main window<br>
05.05.2022 - refactored the application to read <code>vasprun.xml</code><br>
21.03.2022 - incorporated exception handling into vasp-dos-tools and refactored frontend to
display them
"""

setup(
    name="vaspplotsuite",
    packages=find_packages(),
    package_data={"": ["*.png", "*.ttf"]},
    entry_points={"console_scripts": ["vaspplot=vaspplotsuite:run"]},
    version="1.0.6",
    license="MIT",
    description="A program for analysis and plotting of electronic from VASP output data",
    long_description=description,
    long_description_content_type="text/markdown",
    url="https://github.com/adam-bialy/vasp-plot-suite",
    author="Adam Grzelak",
    author_email="contact@adamgrzelak.com",
    keywords=["VASP", "eDOS", "band structure", "density of states"],
    install_requires=["PyQt6", "matplotlib", "numpy", "scipy", "lxml"],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ]
)